@@ -14,9 +14,7 @@
 #ifndef _DIRENT
 #define _DIRENT
 
-#ifndef _TCL
-#include <tcl.h>
-#endif
+#include "tcl.h"
 
 /*
  * Dirent structure, which holds information about a single
@@ -50,8 +48,8 @@ typedef struct _dirdesc {
  * Procedures defined for reading directories:
  */
 
-extern void		closedir _ANSI_ARGS_((DIR *dirp));
-extern DIR *		opendir _ANSI_ARGS_((char *name));
-extern struct dirent *	readdir _ANSI_ARGS_((DIR *dirp));
+extern void		closedir (DIR *dirp);
+extern DIR *		opendir (char *name);
+extern struct dirent *	readdir (DIR *dirp);
 
 #endif /* _DIRENT */
