@@ -13,17 +13,38 @@
 
 #include "tclInt.h"
 
-TclStubs *tclStubsPtr = NULL;
-TclPlatStubs *tclPlatStubsPtr = NULL;
-TclIntStubs *tclIntStubsPtr = NULL;
-TclIntPlatStubs *tclIntPlatStubsPtr = NULL;
-TclTomMathStubs* tclTomMathStubsPtr = NULL;
+MODULE_SCOPE const TclStubs *tclStubsPtr;
+MODULE_SCOPE const TclPlatStubs *tclPlatStubsPtr;
+MODULE_SCOPE const TclIntStubs *tclIntStubsPtr;
+MODULE_SCOPE const TclIntPlatStubs *tclIntPlatStubsPtr;
 
+const TclStubs *tclStubsPtr = NULL;
+const TclPlatStubs *tclPlatStubsPtr = NULL;
+const TclIntStubs *tclIntStubsPtr = NULL;
+const TclIntPlatStubs *tclIntPlatStubsPtr = NULL;
+
+static const TclStubs *
+HasStubSupport(
+    Tcl_Interp *interp)
+{
+    Interp *iPtr = (Interp *) interp;
+
+    if (iPtr->stubTable && (iPtr->stubTable->magic == TCL_STUB_MAGIC)) {
+	return iPtr->stubTable;
+    }
+    iPtr->result = (char *) "interpreter uses an incompatible stubs mechanism";
+    iPtr->freeProc = TCL_STATIC;
+    return NULL;
+}
+
 /*
- * Use our own ISDIGIT to avoid linking to libc on windows
+ * Use our own isdigit to avoid linking to libc on windows
  */
 
-#define ISDIGIT(c) (((unsigned)((c)-'0')) <= 9)
+static int isDigit(const int c)
+{
+    return (c >= '0' && c <= '9');
+}
 
 /*
  *----------------------------------------------------------------------
@@ -43,16 +64,15 @@ TclTomMathStubs* tclTomMathStubsPtr = NULL;
  *----------------------------------------------------------------------
  */
 #undef Tcl_InitStubs
-CONST char *
+MODULE_SCOPE const char *
 Tcl_InitStubs(
     Tcl_Interp *interp,
-    CONST char *version,
+    const char *version,
     int exact)
 {
-    Interp *iPtr = (Interp *) interp;
-    CONST char *actualVersion = NULL;
+    const char *actualVersion = NULL;
     ClientData pkgData = NULL;
-    TclStubs *stubsPtr = iPtr->stubTable;
+    const TclStubs *stubsPtr;
 
     /*
      * We can't optimize this check by caching tclStubsPtr because that
@@ -60,9 +80,8 @@ Tcl_InitStubs(
      * times. [Bug 615304]
      */
 
-    if (!stubsPtr || (stubsPtr->magic != TCL_STUB_MAGIC)) {
-	iPtr->result = "interpreter uses an incompatible stubs mechanism";
-	iPtr->freeProc = TCL_STATIC;
+    stubsPtr = HasStubSupport(interp);
+    if (!stubsPtr) {
 	return NULL;
     }
 
@@ -71,20 +90,20 @@ Tcl_InitStubs(
 	return NULL;
     }
     if (exact) {
-	CONST char *p = version;
+	const char *p = version;
 	int count = 0;
 
 	while (*p) {
-	    count += !ISDIGIT(*p++);
+	    count += !isDigit(*p++);
 	}
 	if (count == 1) {
-	    CONST char *q = actualVersion;
+	    const char *q = actualVersion;
 
 	    p = version;
 	    while (*p && (*p == *q)) {
 		p++; q++;
 	    }
-	    if (*p || ISDIGIT(*q)) {
+	    if (*p || isDigit(*q)) {
 		/* Construct error message */
 		stubsPtr->tcl_PkgRequireEx(interp, "Tcl", version, 1, NULL);
 		return NULL;
@@ -111,61 +130,6 @@ Tcl_InitStubs(
     return actualVersion;
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * TclTomMathInitStubs --
- *
- *	Initializes the Stubs table for Tcl's subset of libtommath
- *
- * Results:
- *	Returns a standard Tcl result.
- *
- * This procedure should not be called directly, but rather through
- * the TclTomMath_InitStubs macro, to insure that the Stubs table
- * matches the header files used in compilation.
- *
- *----------------------------------------------------------------------
- */
-
-#undef TclTomMathInitializeStubs
-
-CONST char*
-TclTomMathInitializeStubs(
-    Tcl_Interp* interp,		/* Tcl interpreter */
-    CONST char* version,	/* Tcl version needed */
-    int epoch,			/* Stubs table epoch from the header files */
-    int revision		/* Stubs table revision number from the
-				 * header files */
-) {
-    int exact = 0;
-    const char* packageName = "tcl::tommath";
-    const char* errMsg = NULL;
-    ClientData pkgClientData = NULL;
-    const char* actualVersion = 
-	tclStubsPtr->tcl_PkgRequireEx(interp, packageName, version, exact, &pkgClientData);
-    TclTomMathStubs* stubsPtr = (TclTomMathStubs*) pkgClientData;
-    if (actualVersion == NULL) {
-	return NULL;
-    }
-    if (pkgClientData == NULL) {
-	errMsg = "missing stub table pointer";
-    } else if ((stubsPtr->tclBN_epoch)() != epoch) {
-	errMsg = "epoch number mismatch";
-    } else if ((stubsPtr->tclBN_revision)() != revision) {
-	errMsg = "requires a later revision";
-    } else {
-	tclTomMathStubsPtr = stubsPtr;
-	return actualVersion;
-    }
-    tclStubsPtr->tcl_ResetResult(interp);
-    tclStubsPtr->tcl_AppendResult(interp, "error loading ", packageName,
-		     " (requested version ", version,
-		     ", actual version ", actualVersion,
-		     "): ", errMsg, NULL);
-    return NULL;
-}
-
 /*
  * Local Variables:
  * mode: c
