@@ -36,10 +36,10 @@
  * 'length' stay aligned.
  */
 
-#define PadBuffer(buffer, length, size)             \
-    if (((length) % (size))) {                      \
-	(buffer) += ((size) - ((length) % (size))); \
-	(length) += ((size) - ((length) % (size))); \
+#define PadBuffer(buffer, length, size)			\
+    if (((length) % (size))) {				\
+	(buffer) += ((size) - ((length) % (size)));	\
+	(length) += ((size) - ((length) % (size)));	\
     }
 
 /*
@@ -961,6 +961,14 @@ CopyString(
 }
 #endif /* NEED_COPYSTRING */
 
+/*
+ * Local Variables:
+ * mode: c
+ * c-basic-offset: 4
+ * fill-column: 78
+ * End:
+ */
+
 /*
  *------------------------------------------------------------------------
  *
@@ -987,24 +995,17 @@ TclWinCPUID(
 
     /* See: <http://en.wikipedia.org/wiki/CPUID> */
 #if defined(HAVE_CPUID)
-#if defined(__x86_64__) || defined(_M_AMD64) || defined (_M_X64)
-    __asm__ __volatile__("movq %%rbx, %%rsi     \n\t" /* save %rbx */
+    __asm__ __volatile__("mov %%ebx, %%edi     \n\t" /* save %ebx */
                  "cpuid            \n\t"
-                 "xchgq %%rsi, %%rbx   \n\t" /* restore the old %rbx */
+                 "mov %%ebx, %%esi   \n\t" /* save what cpuid just put in %ebx */
+                 "mov %%edi, %%ebx  \n\t" /* restore the old %ebx */
                  : "=a"(regsPtr[0]), "=S"(regsPtr[1]), "=c"(regsPtr[2]), "=d"(regsPtr[3])
-                 : "a"(index));
-#else
-    __asm__ __volatile__("mov %%ebx, %%esi     \n\t" /* save %ebx */
-                 "cpuid            \n\t"
-                 "xchg %%esi, %%ebx   \n\t" /* restore the old %ebx */
-                 : "=a"(regsPtr[0]), "=S"(regsPtr[1]), "=c"(regsPtr[2]), "=d"(regsPtr[3])
-                 : "a"(index));
-#endif
+                 : "a"(index) : "edi");
     status = TCL_OK;
 #endif
     return status;
 }
-
+
 /*
  * Local Variables:
  * mode: c
